<?php

namespace Drupal\fleetview_client\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\fleetview_client\Services\FleetviewClientEncryption;
use Drupal\Core\Render\Element\StatusReport;
use Drupal\Core\State\StateInterface;
use Drupal\system\Controller\SystemInfoController;
use Drupal\update\UpdateManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Fleetview client controller.
 */
class FleetviewClientController extends ControllerBase {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The system status encrypt service.
   *
   * @var \Drupal\fleetview_client\Services\FleetviewClientEncryption
   */
  protected $encrypt;

  /**
   * Stores the state storage service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('theme_handler'),
      $container->get('fleetview_client.encrypt'),
      $container->get('update.manager'),
      $container->get('state')
    );
  }

  /**
   * FleetviewClientController constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler.
   * @param \Drupal\fleetview_client\Services\FleetviewClientEncryption $encrypt
   *   The system status encrypt service.
   * @param \Drupal\update\UpdateManagerInterface $update_manager
   *   The update manager.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state storage service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, ThemeHandlerInterface $theme_handler, FleetviewClientEncryption $encrypt, UpdateManagerInterface $update_manager, StateInterface $state) {
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
    $this->encrypt = $encrypt;
    $this->updateManager = $update_manager;
    $this->state = $state;
  }

  /**
   * Loads the route.
   */
  public function load($token) {

    // Add module update data - refresh if not available.
    $this->updateManager->refreshUpdateData();
    if ($available = update_get_available(TRUE)) {
      $this->moduleHandler()->loadInclude('update', 'compare.inc');
      $response['update_data'] = update_calculate_project_data($available);
    }

    // Add cron last run time.
    $response['cron_last_run'] = $this->state->get('system.cron_last');

    // Add status report data.
    $system_manager = \Drupal::service('system.manager');
    $system_info_controller = new SystemInfoController($system_manager);
    $status_report = $system_info_controller->status();

    $status_report['#priorities'] = [
      'warning',
      'checked',
      'ok',
      'error',
    ];
    $status_report = StatusReport::preRenderGroupRequirements($status_report);
    $response['status_report'] = json_encode($status_report);

    $config = $this->config('fleetview_client.settings');

    if (function_exists('openssl_random_pseudo_bytes')) {
      $response = FleetviewClientEncryption::encryptOpenssl(json_encode(["fleetview_client" => $response]));
      return new JsonResponse([
        "fleetview_client" => "encrypted_openssl",
        "data" => $response,
        "drupal_version" => "8",
        "engine_version" => "DRUPAL8",
        "php_version" => phpversion(),
      ]);
    }

  }

  /**
   * Manually checks the update status without the use of cron.
   */
  public function updateStatusManually($token) {
    $this->updateManager->refreshUpdateData();
    $batch_builder = (new BatchBuilder())
      ->setTitle(t('Checking available update data'))
      ->addOperation([$this->updateManager, 'fetchDataBatch'], [])
      ->setProgressMessage(t('Trying to check available update data ...'))
      ->setErrorMessage(t('Error checking available update data.'))
      ->setFinishCallback('update_fetch_data_finished');
    batch_set($batch_builder->toArray());
    batch_process();
    return new JsonResponse([
      "fleetview_client" => 'updated',
      "drupal_version" => "8",
      "engine_version" => "DRUPAL8",
      "php_version" => phpversion(),
    ]);

  }

  /**
   * {@inheritdoc}
   */
  public function access($token) {
    $storedTtoken = $this->config('fleetview_client.settings')->get('fleetview_client_token');
    if ($token == $storedTtoken) {
      return AccessResult::allowed();
    }
    else {
      return AccessResult::forbidden();
    }
  }

}
